/*******************************************************************************
  Sample Application

  File Name:
    app.c

  Summary:


  Description:

 *******************************************************************************/


// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************
#include "app.h"
#include "system_definitions.h"
#if !defined(SYS_CMD_ENABLE)
#include "system/command/sys_command.h"
#endif
#include "system/console/sys_console.h"
//#include <stdio.h>

// *****************************************************************************
// *****************************************************************************
// Section: Global Variable Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    Application strings and buffers are be defined outside this structure.
*/
APP_DATA appData;

BSP_LED_STATE LEDstate = BSP_LED_STATE_OFF;
// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize( void )
{
    /* Initialize the app state to wait for
     * media attach. */
    appData.state = APP_MOUNT_DISK;

    /* Initialize OLED */
    oledInitialize();
}


/*******************************************************************************
  Function:
    void APP_Tasks( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks( void )
{
    SYS_STATUS          tcpipStat;
    TCPIP_NET_HANDLE    netH;
    int                 nNets;
    static IPV4_ADDR    dwLastIP[2] = { {-1}, {-1} };
    IPV4_ADDR           ipAddr;
    int                 i;
    const char          *netName, *netBiosName;
	static uint32_t     startTick=0;
    char LCDBuffer [22];

    switch(appData.state)
    {
        case APP_MOUNT_DISK:
            if(SYS_FS_Mount(SYS_FS_SD_VOL, LOCAL_WEBSITE_PATH_FS, FAT, 0, NULL) == 0)
            {
                SYS_CONSOLE_PRINT("SYS_Initialize: The %s File System is mounted.\r\n", SYS_FS_FATFS_STRING);
                appData.state = APP_TCPIP_WAIT_INIT;

            }
            else
            {
                // SPP +
                static int Err=0, Timeout=0;
                if (!Timeout)
                {
                    oledDrawString (0*8, 0, "     No SD card!     ", Err);
                    oledUpdate ();
                    Err = !Err;
                    Timeout = 20000;
                }
                else
                    Timeout--;
                // SPP -
 //               SYS_CONSOLE_PRINT("SYS_Initialize: Mount the %s File System: pending! \r\n", SYS_FS_FATFS_STRING);
            }
            break;

        case APP_TCPIP_WAIT_INIT:
            tcpipStat = TCPIP_STACK_Status(sysObj.tcpip);
            if(tcpipStat < 0)
            {   // some error occurred
                SYS_CONSOLE_MESSAGE(" APP: TCP/IP stack initialization failed!\r\n");
                // SPP +
                oledClear();
                oledDrawString (0*8, 0, "  APP: TCP/IP stack  ", false);
                oledDrawString (1*8, 0, "initialization failed", false);
                oledUpdate();
                // SPP -
                
                appData.state = APP_TCPIP_ERROR;
            }
            else if(tcpipStat == SYS_STATUS_READY)
            {
                // now that the stack is ready we can check the
                // available interfaces and register
                // a Bonjour service

                nNets = TCPIP_STACK_NumberOfNetworksGet();

                for(i = 0; i < nNets; i++)
                {
                    netH = TCPIP_STACK_IndexToNet(i);
                    netName = TCPIP_STACK_NetNameGet(netH);
                    netBiosName = TCPIP_STACK_NetBIOSName(netH);

#if defined(TCPIP_STACK_USE_NBNS)
                    SYS_CONSOLE_PRINT("    Interface %s on host %s - NBNS enabled\r\n", netName, netBiosName);
#else
                    SYS_CONSOLE_PRINT("    Interface %s on host %s - NBNS disabled\r\n", netName, netBiosName);
#endif  // defined(TCPIP_STACK_USE_NBNS)

#if defined (TCPIP_STACK_USE_ZEROCONF_MDNS_SD)
                    char mDNSServiceName[] = "MyWebServiceNameX "; // base name of the service Must not exceed 16 bytes long
                    // the last digit will be incremented by interface

                    mDNSServiceName[sizeof(mDNSServiceName) - 2] = '1' + i;
                    TCPIP_MDNS_ServiceRegister( netH
                            , mDNSServiceName                   // name of the service
                            ,"_http._tcp.local"                 // type of the service
                            ,80                                 // TCP or UDP port, at which this service is available
                            ,((const uint8_t *)"path=/index.htm")  // TXT info
                            ,1                                  // auto rename the service when if needed
                            ,NULL                               // no callback function
                            ,NULL);                             // no application context
#endif //TCPIP_STACK_USE_ZEROCONF_MDNS_SD

                }
                // SPP +
                oledClear();
                oledDrawString (6*8, 0, "To change this text", false);
                oledDrawString (7*8, 0, "go to: \"forms.htm\"", false);
                oledUpdate();
                // SPP -

                appData.state = APP_TCPIP_TRANSACT;
            }

            break;

        case APP_TCPIP_TRANSACT:

            if(SYS_TMR_TickCountGet() - startTick >= SYS_TMR_TickCounterFrequencyGet()/2ul)
            {
                startTick = SYS_TMR_TickCountGet();
                LEDstate ^= BSP_LED_STATE_ON;
                BSP_LEDStateSet(APP_TCPIP_LED_3, LEDstate);
                // SPP +
                // LEDs and buttons state visualized on the display
                oledClearRow (2, false);
                sprintf (LCDBuffer, " LED1 ");
                oledDrawString (2*8, 0, LCDBuffer, BSP_LEDStateGet(BSP_LED_1));
                sprintf (LCDBuffer, " LED2 ");
                oledDrawString (2*8, 46, LCDBuffer, BSP_LEDStateGet(BSP_LED_2));
                sprintf (LCDBuffer, " LED3 ");
                oledDrawString (2*8, 92, LCDBuffer, BSP_LEDStateGet(BSP_LED_3));

                oledClearRow (4, false);
                sprintf (LCDBuffer, " BUT1 ");
                oledDrawString (4*8, 0, LCDBuffer, BSP_SwitchStateGet(BSP_SWITCH_1));
                sprintf (LCDBuffer, " BUT2 ");
                oledDrawString (4*8, 46, LCDBuffer, BSP_SwitchStateGet(BSP_SWITCH_2));
                sprintf (LCDBuffer, " BUT3 ");
                oledDrawString (4*8, 92, LCDBuffer, BSP_SwitchStateGet(BSP_SWITCH_3));
                oledUpdate();
                // SPP -
            }

            // if the IP address of an interface has changed
            // display the new value on the system console
            nNets = TCPIP_STACK_NumberOfNetworksGet();

            for (i = 0; i < nNets; i++)
            {
                netH = TCPIP_STACK_IndexToNet(i);
                ipAddr.Val = TCPIP_STACK_NetAddress(netH);
                if(dwLastIP[i].Val != ipAddr.Val)
                {
                    dwLastIP[i].Val = ipAddr.Val;

                    SYS_CONSOLE_MESSAGE(TCPIP_STACK_NetNameGet(netH));
                    SYS_CONSOLE_MESSAGE(" IP Address: ");
                    SYS_CONSOLE_PRINT("%d.%d.%d.%d \r\n", ipAddr.v[0], ipAddr.v[1], ipAddr.v[2], ipAddr.v[3]);

                    // SPP +
                    oledClearRow (0, false);
                    sprintf(LCDBuffer, "IP: %d.%d.%d.%d", ipAddr.v[0], ipAddr.v[1], ipAddr.v[2], ipAddr.v[3]);
                    oledDrawString (0, 0, LCDBuffer, false);
                    oledUpdate();
                    // SPP -
                }
            }
            SYS_CMD_READY_TO_READ();
            break;

        default:
            break;
    }
}

/*******************************************************************************
 End of File
*/

