/*******************************************************************************
  ADC Driver Interface Declarations for Static Single Instance Driver

  Company:
    Microchip Technology Inc.

  File Name:
    drv_adc_static.h

  Summary:
    ADC driver interface declarations for the static single instance driver.

  Description:
    The ADC device driver provides a simple interface to manage the ADC
    modules on Microchip microcontrollers. This file defines the interface
    Declarations for the ADC driver.
    
  Remarks:
    Static interfaces incorporate the driver instance number within the names
    of the routines, eliminating the need for an object ID or object handle.
    
    Static single-open interfaces also eliminate the need for the open handle.
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
//DOM-IGNORE-END

#ifndef _DRV_ADC_STATIC_H
#define _DRV_ADC_STATIC_H

#include <stdbool.h>
#include "system_config.h"
#include "peripheral/adc/plib_adc.h"
<#if CONFIG_DRV_ADC_INTERRUPT_MODE == true>
#include "peripheral/int/plib_int.h"
</#if>

typedef enum {

    DRV_ADC_ID_1 = ADC_ID_1,
    DRV_ADC_NUMBER_OF_MODULES

} DRV_ADC_MODULE_ID;

typedef enum {

    DRV_ADC_MUX_A = ADC_MUX_A,
    DRV_ADC_MUX_B = ADC_MUX_B

} DRV_ADC_MUX;

typedef enum {

    DRV_ADC_FILLING_BUF_0TO7 = ADC_FILLING_BUF_0TO7,
    DRV_ADC_FILLING_BUF_8TOF = ADC_FILLING_BUF_8TOF

} DRV_ADC_RESULT_BUF_STATUS;

typedef enum {

    DRV_ADC_REFERENCE_VDD_TO_AVSS = ADC_REFERENCE_VDD_TO_AVSS,
    DRV_ADC_REFERENCE_VREFPLUS_TO_AVSS = ADC_REFERENCE_VREFPLUS_TO_AVSS,
    DRV_ADC_REFERENCE_AVDD_TO_VREF_NEG = ADC_REFERENCE_AVDD_TO_VREF_NEG,
    DRV_ADC_REFERENCE_VREFPLUS_TO_VREFNEG = ADC_REFERENCE_VREFPLUS_TO_VREFNEG

} DRV_ADC_VOLTAGE_REFERENCE;

typedef enum {

    DRV_ADC_SAMPLING_MODE_ALTERNATE_INPUT = ADC_SAMPLING_MODE_ALTERNATE_INPUT,
    DRV_ADC_SAMPLING_MODE_MUXA = ADC_SAMPLING_MODE_MUXA

} DRV_ADC_SAMPLING_MODE;

typedef enum {
    DRV_ADC_1SAMPLE_PER_INTERRUPT = ADC_1SAMPLE_PER_INTERRUPT,
    DRV_ADC_2SAMPLES_PER_INTERRUPT = ADC_2SAMPLES_PER_INTERRUPT,
    DRV_ADC_3SAMPLES_PER_INTERRUPT = ADC_3SAMPLES_PER_INTERRUPT,
    DRV_ADC_4SAMPLES_PER_INTERRUPT = ADC_4SAMPLES_PER_INTERRUPT,
    DRV_ADC_5SAMPLES_PER_INTERRUPT = ADC_5SAMPLES_PER_INTERRUPT,
    DRV_ADC_6SAMPLES_PER_INTERRUPT = ADC_6SAMPLES_PER_INTERRUPT,
    DRV_ADC_7SAMPLES_PER_INTERRUPT = ADC_7SAMPLES_PER_INTERRUPT,
    DRV_ADC_8SAMPLES_PER_INTERRUPT = ADC_8SAMPLES_PER_INTERRUPT,
    DRV_ADC_9SAMPLES_PER_INTERRUPT = ADC_9SAMPLES_PER_INTERRUPT,
    DRV_ADC_10SAMPLES_PER_INTERRUPT = ADC_10SAMPLES_PER_INTERRUPT,
    DRV_ADC_11SAMPLES_PER_INTERRUPT = ADC_11SAMPLES_PER_INTERRUPT,
    DRV_ADC_12SAMPLES_PER_INTERRUPT = ADC_12SAMPLES_PER_INTERRUPT,
    DRV_ADC_13SAMPLES_PER_INTERRUPT = ADC_13SAMPLES_PER_INTERRUPT,
    DRV_ADC_14SAMPLES_PER_INTERRUPT = ADC_14SAMPLES_PER_INTERRUPT,
    DRV_ADC_15SAMPLES_PER_INTERRUPT = ADC_15SAMPLES_PER_INTERRUPT,
    DRV_ADC_16SAMPLES_PER_INTERRUPT = ADC_16SAMPLES_PER_INTERRUPT

} DRV_ADC_SAMPLES_PER_INTERRUPT;

typedef enum {

<#macro ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN0">
    DRV_ADC_INPUT_POSITIVE_AN0 = ADC_INPUT_POSITIVE_AN0,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN1">
    DRV_ADC_INPUT_POSITIVE_AN1 = ADC_INPUT_POSITIVE_AN1,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN2">	
    DRV_ADC_INPUT_POSITIVE_AN2 = ADC_INPUT_POSITIVE_AN2,
</#if>	
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN3">	
    DRV_ADC_INPUT_POSITIVE_AN3 = ADC_INPUT_POSITIVE_AN3,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN4">	
    DRV_ADC_INPUT_POSITIVE_AN4 = ADC_INPUT_POSITIVE_AN4,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN5">	
    DRV_ADC_INPUT_POSITIVE_AN5 = ADC_INPUT_POSITIVE_AN5,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN6">	
    DRV_ADC_INPUT_POSITIVE_AN6 = ADC_INPUT_POSITIVE_AN6,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN7">	
    DRV_ADC_INPUT_POSITIVE_AN7 = ADC_INPUT_POSITIVE_AN7,
</#if>	
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN8">
    DRV_ADC_INPUT_POSITIVE_AN8 = ADC_INPUT_POSITIVE_AN8,
</#if>	
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN9">	
    DRV_ADC_INPUT_POSITIVE_AN9 = ADC_INPUT_POSITIVE_AN9,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN10">
    DRV_ADC_INPUT_POSITIVE_AN10 = ADC_INPUT_POSITIVE_AN10,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN11">	
    DRV_ADC_INPUT_POSITIVE_AN11 = ADC_INPUT_POSITIVE_AN11,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN12">	
    DRV_ADC_INPUT_POSITIVE_AN12 = ADC_INPUT_POSITIVE_AN12,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN13">	
    DRV_ADC_INPUT_POSITIVE_AN13 = ADC_INPUT_POSITIVE_AN13,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN14">	
    DRV_ADC_INPUT_POSITIVE_AN14 = ADC_INPUT_POSITIVE_AN14,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN15">	
    DRV_ADC_INPUT_POSITIVE_AN15 = ADC_INPUT_POSITIVE_AN15,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN16">	
    DRV_ADC_INPUT_POSITIVE_AN16 = ADC_INPUT_POSITIVE_AN16,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN17">	
    DRV_ADC_INPUT_POSITIVE_AN17 = ADC_INPUT_POSITIVE_AN17,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN18">	
    DRV_ADC_INPUT_POSITIVE_AN18 = ADC_INPUT_POSITIVE_AN18,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN19">	
    DRV_ADC_INPUT_POSITIVE_AN19 = ADC_INPUT_POSITIVE_AN19,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN20">	
    DRV_ADC_INPUT_POSITIVE_AN20 = ADC_INPUT_POSITIVE_AN20,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN21">	
    DRV_ADC_INPUT_POSITIVE_AN21 = ADC_INPUT_POSITIVE_AN21,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN22">	
    DRV_ADC_INPUT_POSITIVE_AN22 = ADC_INPUT_POSITIVE_AN22,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN23">	
    DRV_ADC_INPUT_POSITIVE_AN23 = ADC_INPUT_POSITIVE_AN23,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN24">	
    DRV_ADC_INPUT_POSITIVE_AN24 = ADC_INPUT_POSITIVE_AN24,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN25">	
    DRV_ADC_INPUT_POSITIVE_AN25 = ADC_INPUT_POSITIVE_AN25,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN26">
    DRV_ADC_INPUT_POSITIVE_AN26 = ADC_INPUT_POSITIVE_AN26,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN27">	
    DRV_ADC_INPUT_POSITIVE_AN27 = ADC_INPUT_POSITIVE_AN27,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN28">	
    DRV_ADC_INPUT_POSITIVE_AN28 = ADC_INPUT_POSITIVE_AN28,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN29">	
    DRV_ADC_INPUT_POSITIVE_AN29 = ADC_INPUT_POSITIVE_AN29,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN30">	
    DRV_ADC_INPUT_POSITIVE_AN30 = ADC_INPUT_POSITIVE_AN30,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN31">	
    DRV_ADC_INPUT_POSITIVE_AN31 = ADC_INPUT_POSITIVE_AN31,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN32">	
    DRV_ADC_INPUT_POSITIVE_AN32 = ADC_INPUT_POSITIVE_AN32,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN33">	
    DRV_ADC_INPUT_POSITIVE_AN33 = ADC_INPUT_POSITIVE_AN33,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN34">	
    DRV_ADC_INPUT_POSITIVE_AN34 = ADC_INPUT_POSITIVE_AN34,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN35">	
    DRV_ADC_INPUT_POSITIVE_AN35 = ADC_INPUT_POSITIVE_AN35,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN36">	
    DRV_ADC_INPUT_POSITIVE_AN36 = ADC_INPUT_POSITIVE_AN36,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN37">	
    DRV_ADC_INPUT_POSITIVE_AN37 = ADC_INPUT_POSITIVE_AN37,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN38">	
    DRV_ADC_INPUT_POSITIVE_AN38 = ADC_INPUT_POSITIVE_AN38,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN39">	
    DRV_ADC_INPUT_POSITIVE_AN39 = ADC_INPUT_POSITIVE_AN39,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN40">	
    DRV_ADC_INPUT_POSITIVE_AN40 = ADC_INPUT_POSITIVE_AN40,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN41">	
    DRV_ADC_INPUT_POSITIVE_AN41 = ADC_INPUT_POSITIVE_AN41,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN42">	
    DRV_ADC_INPUT_POSITIVE_AN42 = ADC_INPUT_POSITIVE_AN42,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN43">	
    DRV_ADC_INPUT_POSITIVE_AN43 = ADC_INPUT_POSITIVE_AN43,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN44">	
    DRV_ADC_INPUT_POSITIVE_AN44 = ADC_INPUT_POSITIVE_AN44,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN45">	
    DRV_ADC_INPUT_POSITIVE_AN45 = ADC_INPUT_POSITIVE_AN45,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN46">	
    DRV_ADC_INPUT_POSITIVE_AN46 = ADC_INPUT_POSITIVE_AN46,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN47">	
    DRV_ADC_INPUT_POSITIVE_AN47 = ADC_INPUT_POSITIVE_AN47,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN48">	
    DRV_ADC_INPUT_POSITIVE_AN48 = ADC_INPUT_POSITIVE_AN48,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN49">	
    DRV_ADC_INPUT_POSITIVE_AN49 = ADC_INPUT_POSITIVE_AN49,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN50">	
    DRV_ADC_INPUT_POSITIVE_AN50 = ADC_INPUT_POSITIVE_AN50,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN51">	
    DRV_ADC_INPUT_POSITIVE_AN51 = ADC_INPUT_POSITIVE_AN51,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN52">	
    DRV_ADC_INPUT_POSITIVE_AN52 = ADC_INPUT_POSITIVE_AN52,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN53">	
    DRV_ADC_INPUT_POSITIVE_AN53 = ADC_INPUT_POSITIVE_AN53,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN54">	
    DRV_ADC_INPUT_POSITIVE_AN54 = ADC_INPUT_POSITIVE_AN54,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN55">	
    DRV_ADC_INPUT_POSITIVE_AN55 = ADC_INPUT_POSITIVE_AN55,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN56">	
    DRV_ADC_INPUT_POSITIVE_AN56 = ADC_INPUT_POSITIVE_AN56,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN57">	
    DRV_ADC_INPUT_POSITIVE_AN57 = ADC_INPUT_POSITIVE_AN57,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN58">	
    DRV_ADC_INPUT_POSITIVE_AN58 = ADC_INPUT_POSITIVE_AN58,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN59">	
    DRV_ADC_INPUT_POSITIVE_AN59 = ADC_INPUT_POSITIVE_AN59,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN60">	
    DRV_ADC_INPUT_POSITIVE_AN60 = ADC_INPUT_POSITIVE_AN60,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN61">	
    DRV_ADC_INPUT_POSITIVE_AN61 = ADC_INPUT_POSITIVE_AN61,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN62">	
    DRV_ADC_INPUT_POSITIVE_AN62 = ADC_INPUT_POSITIVE_AN62,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_AN63">	
    DRV_ADC_INPUT_POSITIVE_AN63 = ADC_INPUT_POSITIVE_AN63,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_CTMU">	
    DRV_ADC_INPUT_POSITIVE_CTMU = ADC_INPUT_POSITIVE_CTMU,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_IVREF">	
    DRV_ADC_INPUT_POSITIVE_IVREF = ADC_INPUT_POSITIVE_IVREF,
</#if>
<#if DEDICATED_CHN_NUM = "ADC_INPUT_POSITIVE_OPEN">	
    DRV_ADC_INPUT_POSITIVE_OPEN = ADC_INPUT_POSITIVE_OPEN
</#if>
</#macro>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX0 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX0/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX1 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX1/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX2 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX2/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX3 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX3/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX4 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX4/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX5 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX5/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX6 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX6/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX7 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX7/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX8 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX8/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX9 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX9/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX10 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX10/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX11 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX11/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX12 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX12/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX13 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX13/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX14 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX14/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX15 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX15/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX16 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX16/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX17 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX17/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX18 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX18/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX19 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX19/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX20 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX20/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX21 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX21/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX22 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX22/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX23 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX23/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX24 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX24/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX25 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX25/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX26 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX26/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX27 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX27/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX28 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX28/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX29 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX29/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX30 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX30/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX31 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX31/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX32 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX32/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX33 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX33/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX34 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX34/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX35 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX35/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX36 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX36/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX37 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX37/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX38 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX38/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX39 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX39/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX40 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX40/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX41 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX41/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX42 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX42/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX43 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX43/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX44 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX44/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX45 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX45/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX46 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX46/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX47 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX47/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX49 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX48/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX49 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX49/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX50 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX50/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX51 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX51/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX52 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX52/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX53 == true>
<@ADC_POSITIVE_INPUT_ENUM DEDICATED_CHN_NUM=CONFIG_DRV_ADC_POSITIVE_CHANNEL_NUMBER_IDX53/>
</#if>
    DRV_ADC_POSITIVE_INPUT_NUM
	
} DRV_ADC_INPUTS_POSITIVE;

typedef enum {

<#macro ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN0">
    DRV_ADC_INPUT_SCAN_AN0 = ADC_INPUT_SCAN_AN0,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN1">
    DRV_ADC_INPUT_SCAN_AN1 = ADC_INPUT_SCAN_AN1,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN2">	
    DRV_ADC_INPUT_SCAN_AN2 = ADC_INPUT_SCAN_AN2,
</#if>	
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN3">	
    DRV_ADC_INPUT_SCAN_AN3 = ADC_INPUT_SCAN_AN3,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN4">	
    DRV_ADC_INPUT_SCAN_AN4 = ADC_INPUT_SCAN_AN4,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN5">	
    DRV_ADC_INPUT_SCAN_AN5 = ADC_INPUT_SCAN_AN5,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN6">	
    DRV_ADC_INPUT_SCAN_AN6 = ADC_INPUT_SCAN_AN6,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN7">	
    DRV_ADC_INPUT_SCAN_AN7 = ADC_INPUT_SCAN_AN7,
</#if>	
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN8">
    DRV_ADC_INPUT_SCAN_AN8 = ADC_INPUT_SCAN_AN8,
</#if>	
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN9">	
    DRV_ADC_INPUT_SCAN_AN9 = ADC_INPUT_SCAN_AN9,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN10">
    DRV_ADC_INPUT_SCAN_AN10 = ADC_INPUT_SCAN_AN10,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN11">	
    DRV_ADC_INPUT_SCAN_AN11 = ADC_INPUT_SCAN_AN11,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN12">	
    DRV_ADC_INPUT_SCAN_AN12 = ADC_INPUT_SCAN_AN12,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN13">	
    DRV_ADC_INPUT_SCAN_AN13 = ADC_INPUT_SCAN_AN13,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN14">	
    DRV_ADC_INPUT_SCAN_AN14 = ADC_INPUT_SCAN_AN14,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN15">	
    DRV_ADC_INPUT_SCAN_AN15 = ADC_INPUT_SCAN_AN15,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN16">	
    DRV_ADC_INPUT_SCAN_AN16 = ADC_INPUT_SCAN_AN16,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN17">	
    DRV_ADC_INPUT_SCAN_AN17 = ADC_INPUT_SCAN_AN17,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN18">	
    DRV_ADC_INPUT_SCAN_AN18 = ADC_INPUT_SCAN_AN18,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN19">	
    DRV_ADC_INPUT_SCAN_AN19 = ADC_INPUT_SCAN_AN19,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN20">	
    DRV_ADC_INPUT_SCAN_AN20 = ADC_INPUT_SCAN_AN20,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN21">	
    DRV_ADC_INPUT_SCAN_AN21 = ADC_INPUT_SCAN_AN21,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN22">	
    DRV_ADC_INPUT_SCAN_AN22 = ADC_INPUT_SCAN_AN22,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN23">	
    DRV_ADC_INPUT_SCAN_AN23 = ADC_INPUT_SCAN_AN23,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN24">	
    DRV_ADC_INPUT_SCAN_AN24 = ADC_INPUT_SCAN_AN24,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN25">	
    DRV_ADC_INPUT_SCAN_AN25 = ADC_INPUT_SCAN_AN25,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN26">
    DRV_ADC_INPUT_SCAN_AN26 = ADC_INPUT_SCAN_AN26,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN27">	
    DRV_ADC_INPUT_SCAN_AN27 = ADC_INPUT_SCAN_AN27,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN28">	
    DRV_ADC_INPUT_SCAN_AN28 = ADC_INPUT_SCAN_AN28,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN29">	
    DRV_ADC_INPUT_SCAN_AN29 = ADC_INPUT_SCAN_AN29,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN30">	
    DRV_ADC_INPUT_SCAN_AN30 = ADC_INPUT_SCAN_AN30,
</#if>
<#if SCAN_CHN_NUM = "ADC_INPUT_SCAN_AN31">	
    DRV_ADC_INPUT_SCAN_AN31 = ADC_INPUT_SCAN_AN31,
</#if>
</#macro>	
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX0 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX0/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX1 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX1/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX2 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX2/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX3 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX3/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX4 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX4/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX5 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX5/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX6 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX6/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX7 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX7/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX8 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX8/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX9 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX9/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX10 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX10/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX11 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX11/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX12 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX12/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX13 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX13/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX14 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX14/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX15 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX15/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX16 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX16/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX17 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX17/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX18 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX18/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX19 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX19/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX20 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX20/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX21 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX21/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX22 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX22/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX23 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX23/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX24 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX24/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX25 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX25/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX26 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX26/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX27 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX27/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX28 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX28/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX29 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX29/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX30 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX30/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX31 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX31/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX32 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX32/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX33 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX33/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX34 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX34/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX35 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX35/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX36 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX36/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX37 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX37/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX38 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX38/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX39 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX39/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX40 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX40/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX41 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX41/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX42 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX42/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX43 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX43/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX44 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX44/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX45 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX45/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX46 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX46/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX47 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX47/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX49 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX48/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX49 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX49/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX50 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX50/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX51 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX51/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX52 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX52/>
</#if>
<#if CONFIG_DRV_ADC_CHANNEL_INST_IDX53 == true>
<@ADC_POSITIVE_INPUT_ENUM SCAN_CHN_NUM=CONFIG_DRV_ADC_SCAN_CHANNEL_NUMBER_IDX53/>
</#if>
    DRV_ADC_SCAN_INPUT_NUM
} DRV_ADC_INPUTS_SCAN;

<#if CONFIG_HAVE_ADC_ESCAN == true>
typedef enum {

    DRV_ADC_INPUT_SCAN_AN32 = ADC_INPUT_SCAN_AN32,
    DRV_ADC_INPUT_SCAN_AN33 = ADC_INPUT_SCAN_AN33,
    DRV_ADC_INPUT_SCAN_AN34 = ADC_INPUT_SCAN_AN34,
    DRV_ADC_INPUT_SCAN_AN35 = ADC_INPUT_SCAN_AN35,
    DRV_ADC_INPUT_SCAN_AN36 = ADC_INPUT_SCAN_AN36,
    DRV_ADC_INPUT_SCAN_AN37 = ADC_INPUT_SCAN_AN37,
    DRV_ADC_INPUT_SCAN_AN38 = ADC_INPUT_SCAN_AN38,
    DRV_ADC_INPUT_SCAN_AN39 = ADC_INPUT_SCAN_AN39,
    DRV_ADC_INPUT_SCAN_AN40 = ADC_INPUT_SCAN_AN40,
    DRV_ADC_INPUT_SCAN_AN41 = ADC_INPUT_SCAN_AN41,
    DRV_ADC_INPUT_SCAN_AN42 = ADC_INPUT_SCAN_AN42,
    DRV_ADC_INPUT_SCAN_AN43 = ADC_INPUT_SCAN_AN43,
    DRV_ADC_INPUT_SCAN_AN44 = ADC_INPUT_SCAN_AN44,
    DRV_ADC_INPUT_SCAN_AN45 = ADC_INPUT_SCAN_AN45,
    DRV_ADC_INPUT_SCAN_AN46 = ADC_INPUT_SCAN_AN46,
    DRV_ADC_INPUT_SCAN_AN47 = ADC_INPUT_SCAN_AN47,
    DRV_ADC_INPUT_SCAN_IVREF = ADC_INPUT_SCAN_IVREF,
    DRV_ADC_INPUT_SCAN_CTMU = ADC_INPUT_SCAN_CTMU,
    DRV_ADC_INPUT_SCAN_VSS = ADC_INPUT_SCAN_VSS

} DRV_ADC_INPUTS_SCAN_EXTENDED;
</#if>

typedef enum {

    DRV_ADC_INPUT_NEGATIVE_VREF_MINUS = ADC_INPUT_NEGATIVE_VREF_MINUS,
    DRV_ADC_INPUT_NEGATIVE_AN1 = ADC_INPUT_NEGATIVE_AN1

} DRV_ADC_INPUTS_NEGATIVE;

typedef enum {

    DRV_ADC_CLOCK_SOURCE_SYSTEM_CLOCK = ADC_CLOCK_SOURCE_SYSTEM_CLOCK,
    DRV_ADC_CLOCK_SOURCE_INTERNAL_RC = ADC_CLOCK_SOURCE_INTERNAL_RC

} DRV_ADC_CLOCK_SOURCE;

typedef enum {

    DRV_ADC_CONVERSION_TRIGGER_SAMP_CLEAR = ADC_CONVERSION_TRIGGER_SAMP_CLEAR,
    DRV_ADC_CONVERSION_TRIGGER_INT0_TRANSITION = ADC_CONVERSION_TRIGGER_INT0_TRANSITION,
    DRV_ADC_CONVERSION_TRIGGER_TMR3_COMPARE_MATCH = ADC_CONVERSION_TRIGGER_TMR3_COMPARE_MATCH,
    DRV_ADC_CONVERSION_TRIGGER_CTMU_EVENT = ADC_CONVERSION_TRIGGER_CTMU_EVENT,
    DRV_ADC_CONVERSION_TRIGGER_INTERNAL_COUNT = ADC_CONVERSION_TRIGGER_INTERNAL_COUNT

} DRV_ADC_CONVERSION_TRIGGER_SOURCE;

typedef enum {

    DRV_ADC_RESULT_FORMAT_INTEGER_16BIT = ADC_RESULT_FORMAT_INTEGER_16BIT,
    DRV_ADC_RESULT_FORMAT_SIGNED_INTEGER_16BIT = ADC_RESULT_FORMAT_SIGNED_INTEGER_16BIT,
    DRV_ADC_RESULT_FORMAT_FRACTIONAL_16BIT = ADC_RESULT_FORMAT_FRACTIONAL_16BIT,
    DRV_ADC_RESULT_FORMAT_SIGNED_FRACTIONAL_16BIT = ADC_RESULT_FORMAT_SIGNED_FRACTIONAL_16BIT,
    DRV_ADC_RESULT_FORMAT_INTEGER_32BIT = ADC_RESULT_FORMAT_INTEGER_32BIT,
    DRV_ADC_RESULT_FORMAT_SIGNED_INTEGER_32BIT = ADC_RESULT_FORMAT_SIGNED_INTEGER_32BIT,
    DRV_ADC_RESULT_FORMAT_FRACTIONAL_32BIT = ADC_RESULT_FORMAT_FRACTIONAL_32BIT,
    DRV_ADC_RESULT_FORMAT_SIGNED_FRACTIONAL_32BIT = ADC_RESULT_FORMAT_SIGNED_FRACTIONAL_32BIT

} DRV_ADC_RESULT_FORMAT;

// *****************************************************************************
// *****************************************************************************
// Section: Interface Headers for ADC Static Driver
// *****************************************************************************
// *****************************************************************************
void DRV_ADC_Initialize(void);

inline void DRV_ADC_DeInitialize(void);

inline void DRV_ADC_Open(void);

inline void DRV_ADC_Close(void);

inline void DRV_ADC_Start(void);

inline void DRV_ADC_Stop(void);

inline void DRV_ADC_NegativeInputSelect(DRV_ADC_MUX mux, DRV_ADC_INPUTS_NEGATIVE input);

inline void DRV_ADC_PositiveInputSelect(DRV_ADC_MUX mux, DRV_ADC_INPUTS_POSITIVE input);

inline void DRV_ADC_ChannelScanInputsAdd(DRV_ADC_INPUTS_SCAN scanInput);

inline void DRV_ADC_ChannelScanInputsRemove(DRV_ADC_INPUTS_SCAN scanInput);
<#if CONFIG_HAVE_ADC_ESCAN == true>

inline void DRV_ADC_ChannelExtendedScanInputsAdd(DRV_ADC_INPUTS_SCAN_EXTENDED eScanInput);

inline void DRV_ADC_ChannelExtendedScanInputsRemove(DRV_ADC_INPUTS_SCAN_EXTENDED eScanInput);
</#if>

ADC_SAMPLE DRV_ADC_SamplesRead(uint8_t bufIndex);

bool DRV_ADC_SamplesAvailable(void);

#endif // #ifndef _DRV_ADC_STATIC_H

/*******************************************************************************
 End of File
*/
