<#--
/*******************************************************************************
  I2C Driver Initialization File

  File Name:
    drv_i2c_init.c.ftl

  Summary:
    This file contains source code necessary to initialize the system.

  Description:
    This file contains source code necessary to initialize the system.  It
    implements the "SYS_Initialize" function, configuration bits, and allocates
    any necessary global system resources, such as the systemObjects structure
    that contains the object handles to all the MPLAB Harmony module objects in
    the system.
 *******************************************************************************/

/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
 -->
<#if CONFIG_DRV_I2C_INST_IDX0 == true>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "DYNAMIC">
    sysObj.drvI2C0 = DRV_I2C_Initialize(DRV_I2C_INDEX_0, (SYS_MODULE_INIT *)&drvI2C0InitData);
</#if>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "STATIC">
    DRV_I2C0_Initialize();
</#if>
</#if>
<#if CONFIG_DRV_I2C_INST_IDX1 == true>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "DYNAMIC">
    sysObj.drvI2C1 = DRV_I2C_Initialize(DRV_I2C_INDEX_1, (SYS_MODULE_INIT *)&drvI2C1InitData);
</#if>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "STATIC">
    DRV_I2C1_Initialize();
</#if>
</#if>
<#if CONFIG_DRV_I2C_INST_IDX2 == true>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "DYNAMIC">
    sysObj.drvI2C2 = DRV_I2C_Initialize(DRV_I2C_INDEX_2, (SYS_MODULE_INIT *)&drvI2C2InitData);
</#if>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "STATIC">
    DRV_I2C2_Initialize();
</#if>
</#if>
<#if CONFIG_DRV_I2C_INST_IDX3 == true>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "DYNAMIC">
    sysObj.drvI2C3 = DRV_I2C_Initialize(DRV_I2C_INDEX_3, (SYS_MODULE_INIT *)&drvI2C3InitData);
</#if>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "STATIC">
    DRV_I2C3_Initialize();
</#if>
</#if>
<#if CONFIG_DRV_I2C_INST_IDX4 == true>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "DYNAMIC">
    sysObj.drvI2C4 = DRV_I2C_Initialize(DRV_I2C_INDEX_4, (SYS_MODULE_INIT *)&drvI2C4InitData);
</#if>
<#if CONFIG_DRV_I2C_DRIVER_MODE == "STATIC">
    DRV_I2C4_Initialize();
</#if>
</#if>

<#if CONFIG_DRV_I2C_DRIVER_MODE == "DYNAMIC">
<#if CONFIG_USE_SYS_INT == true && CONFIG_DRV_I2C_INTERRUPT_MODE == true>

<#if CONFIG_DRV_I2C_INST_IDX0 == true>
<#if CONFIG_DRV_I2C_BIT_BANG_IDX0 == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_BIT_BANG_INT_PRIORITY_IDX0});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_BIT_BANG_INT_SUB_PRIORITY_IDX0});
<#else>	
<#if CONFIG_PIC32MZ == true>
<#if CONFIG_DRV_I2C_OPERATION_MODE_IDX0 = "DRV_I2C_MODE_MASTER">
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_MASTER_INT_PRIORITY_IDX0});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_MASTER_INT_SUB_PRIORITY_IDX0});
<#else>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_SLAVE_INT_PRIORITY_IDX0});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_SLAVE_INT_SUB_PRIORITY_IDX0});
</#if>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_ERR_INT_PRIORITY_IDX0});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_ERR_INT_SUB_PRIORITY_IDX0});
</#if>
<#if CONFIG_PIC32MX == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_INT_PRIORITY_IDX0});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX0}, ${CONFIG_DRV_I2C_INT_SUB_PRIORITY_IDX0});
</#if>
</#if>
</#if>

<#if CONFIG_DRV_I2C_INST_IDX1 == true>
<#if CONFIG_DRV_I2C_BIT_BANG_IDX1 == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_BIT_BANG_INT_PRIORITY_IDX1});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_BIT_BANG_INT_SUB_PRIORITY_IDX1});
<#else>	
<#if CONFIG_PIC32MZ == true>
<#if CONFIG_DRV_I2C_OPERATION_MODE_IDX1 = "DRV_I2C_MODE_MASTER">
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_MASTER_INT_PRIORITY_IDX1});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_MASTER_INT_SUB_PRIORITY_IDX1});
<#else>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_SLAVE_INT_PRIORITY_IDX1});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_SLAVE_INT_SUB_PRIORITY_IDX1});
</#if>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_ERR_INT_PRIORITY_IDX1});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_ERR_INT_SUB_PRIORITY_IDX1});
</#if>
<#if CONFIG_PIC32MX == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_INT_PRIORITY_IDX1});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX1}, ${CONFIG_DRV_I2C_INT_SUB_PRIORITY_IDX1});
</#if>
</#if>
</#if>

<#if CONFIG_DRV_I2C_INST_IDX2 == true>
<#if CONFIG_DRV_I2C_BIT_BANG_IDX2 == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_BIT_BANG_INT_PRIORITY_IDX2});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_BIT_BANG_INT_SUB_PRIORITY_IDX2});
<#else>	
<#if CONFIG_PIC32MZ == true>
<#if CONFIG_DRV_I2C_OPERATION_MODE_IDX2 = "DRV_I2C_MODE_MASTER">
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_MASTER_INT_PRIORITY_IDX2});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_MASTER_INT_SUB_PRIORITY_IDX2});
<#else>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_SLAVE_INT_PRIORITY_IDX2});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_SLAVE_INT_SUB_PRIORITY_IDX2});
</#if>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_ERR_INT_PRIORITY_IDX2});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_ERR_INT_SUB_PRIORITY_IDX2});
</#if>
<#if CONFIG_PIC32MX == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_INT_PRIORITY_IDX2});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX2}, ${CONFIG_DRV_I2C_INT_SUB_PRIORITY_IDX2});
</#if>
</#if>
</#if>
<#if CONFIG_DRV_I2C_INST_IDX3 == true>
<#if CONFIG_DRV_I2C_BIT_BANG_IDX3 == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_BIT_BANG_INT_PRIORITY_IDX3});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_BIT_BANG_INT_SUB_PRIORITY_IDX3});
<#else>	
<#if CONFIG_PIC32MZ == true>
<#if CONFIG_DRV_I2C_OPERATION_MODE_IDX3 = "DRV_I2C_MODE_MASTER">
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_MASTER_INT_PRIORITY_IDX3});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_MASTER_INT_SUB_PRIORITY_IDX3});
<#else>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_SLAVE_INT_PRIORITY_IDX3});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_SLAVE_INT_SUB_PRIORITY_IDX3});
</#if>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_ERR_INT_PRIORITY_IDX3});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_ERR_INT_SUB_PRIORITY_IDX3});
</#if>
<#if CONFIG_PIC32MX == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_INT_PRIORITY_IDX3});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX3}, ${CONFIG_DRV_I2C_INT_SUB_PRIORITY_IDX3});
</#if>
</#if>
</#if>
<#if CONFIG_DRV_I2C_INST_IDX4 == true>
<#if CONFIG_DRV_I2C_BIT_BANG_IDX4 == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_BIT_BANG_INT_PRIORITY_IDX4});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_BIT_BANG_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_BIT_BANG_INT_SUB_PRIORITY_IDX4});
<#else>	
<#if CONFIG_PIC32MZ == true>
<#if CONFIG_DRV_I2C_OPERATION_MODE_IDX4 = "DRV_I2C_MODE_MASTER">
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_MASTER_INT_PRIORITY_IDX4});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_MASTER_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_MASTER_INT_SUB_PRIORITY_IDX4});
<#else>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_SLAVE_INT_PRIORITY_IDX4});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_SLAVE_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_SLAVE_INT_SUB_PRIORITY_IDX4});
</#if>	
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_ERR_INT_PRIORITY_IDX4});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_ERR_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_ERR_INT_SUB_PRIORITY_IDX4});
</#if>
<#if CONFIG_PIC32MX == true>
    SYS_INT_VectorPrioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_INT_PRIORITY_IDX4});
    SYS_INT_VectorSubprioritySet(${CONFIG_DRV_I2C_INT_VECTOR_IDX4}, ${CONFIG_DRV_I2C_INT_SUB_PRIORITY_IDX4});
</#if>
</#if>
</#if>
</#if>
</#if>
<#--
/*******************************************************************************
 End of File
*/-->
