/*******************************************************************************
  tcpip MRF24WN connection manager implementation

  File Name:  
    wdrv_mrf24wn_connmgr.c  
  
  Summary:
    
  Description:
*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/

#include "wdrv_mrf24wn_main.h"

#define DISCONNECT_DONE_NOTIFY() WDRV_SemGive(&g_wdrv_priv.disconnectDoneSync)

extern WDRV_CONFIG g_wdrv_config;
static WDRV_CONNECTION_STATES s_ConnectionStatus = WDRV_CSTATE_NOT_CONNECTED;
static bool s_logicalConnection = false;
static char *s_connect_failure_reason[] = {
    "",
    "NO_NETWORK_AVAIL",
    "LOST_LINK",
    "DISCONNECT_CMD",
    "BSS_DISCONNECTED",
    "AUTH_FAILED",
    "ASSOC_FAILED",
    "NO_RESOURCES_AVAIL",
    "CONNECTION_DENIED",
    "",
    "INVALID_PROFILE",
    "",
    "PROFILE_MISMATCH",
    "CONNECTION_EVICTED",    
};

static void ConnectionStateSet(bool state);
static void ConnectStatusUpdate(bool connected, uint8_t reason);


void ProceedConnectEventCB(int32_t connected, uint8_t devID, uint8_t *bssid, bool bssConn, uint8_t reason)
{
    if (connected == true) {
        ConnectionStateSet(true);
        WDRV_DBG_INFORM_MESSAGE(("Connection succeeded\r\n"));
    } else if (connected == false){             
        ConnectionStateSet(false);
        WDRV_DBG_INFORM_PRINT(("Connection failed - %s\r\n", s_connect_failure_reason[reason]));
    } 

    ConnectStatusUpdate(connected, reason);
}

bool isLinkUp()
{
    return s_logicalConnection;   
}      

static void ConnectionStateSet(bool state)
{
    s_logicalConnection = state;
}


static void NetModeSet(uint8_t networkType)
{
    switch (networkType) {
    case WDRV_NETWORK_TYPE_INFRASTRUCTURE:
        WDRV_EXT_CmdNetModeBSSSet();
        break;
    case WDRV_NETWORK_TYPE_ADHOC:
        WDRV_ASSERT(false, "Adhoc is not supported for now");
        break;
    case WDRV_NETWORK_TYPE_SOFT_AP:
        WDRV_EXT_CmdNetModeAPSet();
        break;
    case WDRV_NETWORK_TYPE_P2P:
        WDRV_ASSERT(false, "P2P is not supported for now");
        break;
    default:
        WDRV_ASSERT(false, "Undefined network type");
        break;
    }
}

static void SecuritySet(uint8_t securityMode)
{
    bool pinMode;

    switch (securityMode) {
    case WDRV_SECURITY_OPEN:
        WDRV_EXT_CmdSecNoneSet();
        break;
    case WDRV_SECURITY_WEP_40:
    case WDRV_SECURITY_WEP_104:
        WDRV_EXT_CmdSecWEPSet(WDRV_CONFIG_PARAMS(SecurityKey), WDRV_CONFIG_PARAMS(SecurityKeyLength));
        break;
    case WDRV_SECURITY_WPA_WITH_PASS_PHRASE:
        WDRV_EXT_CmdSecWPASet(WDRV_CONFIG_PARAMS(SecurityKey), WDRV_CONFIG_PARAMS(SecurityKeyLength));
        break;
    case WDRV_SECURITY_WPA2_WITH_PASS_PHRASE:
        WDRV_EXT_CmdSecWPA2Set(WDRV_CONFIG_PARAMS(SecurityKey), WDRV_CONFIG_PARAMS(SecurityKeyLength));
        break;
    case WDRV_SECURITY_WPS_PIN:
    case WDRV_SECURITY_WPS_PUSH_BUTTON:
         pinMode = WDRV_CONFIG_PARAMS(SecurityMode) == WDRV_SECURITY_WPS_PIN ? true : false;
         WDRV_EXT_CmdSecWpsSet(pinMode, WDRV_CONFIG_PARAMS(SecurityKey), WDRV_CONFIG_PARAMS(SecurityKeyLength));
        break;
    default:
        WDRV_ASSERT(false, "Undefined security mode");
        break;
    }
}

TCPIP_MAC_RES WDRV_Connect(void)
{
    TCPIP_MAC_RES res = TCPIP_MAC_RES_OK;

    WDRV_EXT_CmdSSIDSet(WDRV_CONFIG_PARAMS(netSSID), WDRV_CONFIG_PARAMS(SsidLength));

    NetModeSet(WDRV_CONFIG_PARAMS(networkType));

    if (WDRV_CONFIG_PARAMS(networkType) == WDRV_NETWORK_TYPE_SOFT_AP)
        WDRV_EXT_CmdChannelSet(WDRV_DEFAULT_CHANNEL); 

    SecuritySet(WDRV_CONFIG_PARAMS(SecurityMode));
    WDRV_EXT_CmdPowerSavePut(false);
    WDRV_DBG_INFORM_MESSAGE(("\r\nStart WiFi Connect . . .\r\n"));

    // start the WiFi connection process
    if ((WDRV_CONFIG_PARAMS(SecurityMode) != WDRV_SECURITY_WPS_PIN) &&
        (WDRV_CONFIG_PARAMS(SecurityMode) != WDRV_SECURITY_WPS_PUSH_BUTTON)) {
        if (WDRV_EXT_CmdConnect() == 0)
            s_ConnectionStatus = WDRV_CSTATE_CONNECTION_IN_PROGRESS;
    }

    return res;
}

void WPSDoneCB(void)
{
    uint32_t status;

    WDRV_EXT_WPSResultsRead(p_wdrv_configData, &status);

    if (status == WDRV_SUCCESS) {
        WDRV_DBG_INFORM_MESSAGE(("WPS process finished successfully.\r\n"));
        WDRV_Connect();
    } else {
        WDRV_DBG_INFORM_PRINT(("WPS process did not finish successfully. (%d)\r\n", status));
        /* TODO: handle error case */
    }
}

static void LinkDownTimeoutCallback(uintptr_t context, uint32_t currTick)
{
    if (s_ConnectionStatus == WDRV_CSTATE_CONNECTION_TEMPORARY_LOST) {
        s_ConnectionStatus = WDRV_CSTATE_CONNECTION_PERMANENTLY_LOST;
        WDRV_EXT_CmdDisconnect();
        WDRV_DBG_INFORM_MESSAGE(("Lost connection permanently !\r\n")); 
    }
}

static void ConnectStatusUpdate(bool connected, uint8_t reason)
{
    static SYS_TMR_HANDLE timer = 0;
    uint16_t timeout;

    if (connected == true) {
        if (s_ConnectionStatus == WDRV_CSTATE_CONNECTION_TEMPORARY_LOST)
            g_wdrv_priv.isConnReestablished = true;
        s_ConnectionStatus = WDRV_CSTATE_CONNECTED_INFRASTRUCTURE;
    } else {
        switch (reason) {
        case WDRV_DISCONNECT_REASON_DISCONNECT_CMD:        // = 0x03,
        case WDRV_DISCONNECT_REASON_NO_NETWORK_AVAIL:      // = 0x01
        case WDRV_DISCONNECT_REASON_BSS_DISCONNECTED:      // = 0x04,
        case WDRV_DISCONNECT_REASON_AUTH_FAILED:           // = 0x05,
        case WDRV_DISCONNECT_REASON_ASSOC_FAILED:          // = 0x06,
        case WDRV_DISCONNECT_REASON_NO_RESOURCES_AVAIL:    // = 0x07,
        case WDRV_DISCONNECT_REASON_CONNECTION_DENIED:     // = 0x08,
        case WDRV_DISCONNECT_REASON_INVALID_PROFILE:       // = 0x0A,
        case WDRV_DISCONNECT_REASON_PROFILE_MISMATCH:      // = 0x0C   
        case WDRV_DISCONNECT_REASON_CONNECTION_EVICTED:    // = 0x0D,
            if (g_wdrv_priv.isDisconnectRequested)
                DISCONNECT_DONE_NOTIFY();
            s_ConnectionStatus = WDRV_CSTATE_NOT_CONNECTED;
            break;
        case WDRV_DISCONNECT_REASON_LOST_LINK:             // = 0x02
            s_ConnectionStatus = WDRV_CSTATE_CONNECTION_TEMPORARY_LOST;

            if (timer != 0)
                SYS_TMR_CallbackStop(timer);

            timeout = SYS_TMR_TickCounterFrequencyGet() * 30; // waiting 30 seconds
            timer = SYS_TMR_CallbackSingle(timeout, 0, LinkDownTimeoutCallback);
            break;
        default:
            WDRV_ASSERT(false, "Undefined reason code\r\n");
            break;
        }
    }
}

WDRV_CONNECTION_STATES WDRV_ConnectStatus_Get(void)
{
    return s_ConnectionStatus;
}

//DOM-IGNORE-END
